from django.contrib.sessions.backends.cached_db import (
    SessionStore as DjangoSessionStore,
)

from .db import SessionStore as DBSessionStore

KEY_PREFIX = "django_user_sessions_ng.cached_db"


class SessionStore(DBSessionStore, DjangoSessionStore):
    cache_key_prefix = KEY_PREFIX



from importlib import import_module

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.sessions.base_session import AbstractBaseSession, BaseSessionManager
from django.core.cache import caches
from django.db import models

from django_user_sessions_ng.backends import SessionBaseChild

from .geoip import get_human_readable_location, get_location_data

User = get_user_model()


class SessionQuerySet(models.QuerySet):
    def delete(self) -> tuple[int, dict[str, int]]:
        SessionStore = Session.get_session_store_class()
        if prefix := getattr(SessionStore, "cache_key_prefix", None):
            caches[settings.SESSION_CACHE_ALIAS].delete_many(f"{prefix}{session.session_key}" for session in self)
        return super().delete()


class SessionManager(BaseSessionManager.from_queryset(SessionQuerySet)):
    use_in_migrations = True


class Session(AbstractBaseSession):
    user = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        related_name="sessions",
        db_index=True,
        null=True,
    )
    ip = models.GenericIPAddressField(null=True, blank=True)
    device = models.CharField(max_length=255, null=True)

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    objects = SessionManager()

    @classmethod
    def get_session_store_class(cls) -> SessionBaseChild:
        return import_module(settings.SESSION_ENGINE).SessionStore

    @property
    def location_data(self) -> dict | None:
        return get_location_data(self.ip)

    @property
    def location(self) -> str | None:
        return get_human_readable_location(self.ip)

    def delete(self, using=None, keep_parents: bool = False) -> tuple[int, dict[str, int]]:
        SessionStore = self.get_session_store_class()
        if prefix := getattr(SessionStore, "cache_key_prefix", None):
            caches[settings.SESSION_CACHE_ALIAS].delete(f"{prefix}{self.session_key}")
        return super().delete(using=using, keep_parents=keep_parents)

    class Meta(AbstractBaseSession.Meta):
        db_table = "django_user_sessions_ng_user_sessions"
        verbose_name = "session"
        verbose_name_plural = "sessions"
        ordering = ("-created_at",)



import logging

from django.contrib.gis.geoip2 import HAS_GEOIP2


def get_location_data(ip: str) -> dict | None:
    if not HAS_GEOIP2:
        return logging.error("GeoIP2 is not available.")

    from django.contrib.gis.geoip2 import GeoIP2

    try:
        g = GeoIP2()
    except Exception as e:  # FIXME: too broad exception clause
        return logging.error(str(e))

    try:
        return g.city(ip)
    except Exception:  # FIXME: too broad exception clause
        pass

    try:
        return g.country(ip)
    except Exception as e:  # FIXME: too broad exception clause
        return logging.error(str(e))


def get_human_readable_location(ip: str) -> str | None:
    if loc := get_location_data(ip):
        if loc.get("country_name"):
            if loc.get("city"):
                return f"{loc['city']}, {loc['country_name']}"
            return loc["country_name"]

    return None



# Generated by Django 5.0.4 on 2024-04-27 20:22

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import django_user_sessions_ng.models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Session",
            fields=[
                (
                    "session_key",
                    models.CharField(
                        max_length=40,
                        primary_key=True,
                        serialize=False,
                        verbose_name="session key",
                    ),
                ),
                ("session_data", models.TextField(verbose_name="session data")),
                (
                    "expire_date",
                    models.DateTimeField(db_index=True, verbose_name="expire date"),
                ),
                ("ip", models.GenericIPAddressField(blank=True, null=True)),
                ("device", models.CharField(max_length=255, null=True)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="sessions",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "session",
                "verbose_name_plural": "sessions",
                "db_table": "django_user_sessions_ng_user_sessions",
                "ordering": ("-created_at",),
                "abstract": False,
            },
            managers=[
                ("objects", django_user_sessions_ng.models.SessionManager()),
            ],
        ),
    ]



import json

import pytest
import user_agents
from django.apps import apps
from django.contrib.auth import SESSION_KEY
from django.contrib.auth.models import User

from django_user_sessions_ng import SESSION_DEVICE_KEY, SESSION_IP_KEY

Session = apps.get_model("django_user_sessions_ng", "Session")
USER_AGENT = "Mozilla/5.0 (iPhone; CPU iPhone OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9B179 Safari/7534.48.3"
DEVICE = str(user_agents.parse(USER_AGENT))


@pytest.mark.django_db
def test_unmodified_session(client, settings):
    client.get("/", HTTP_USER_AGENT=USER_AGENT)
    assert settings.SESSION_COOKIE_NAME not in client.cookies


@pytest.mark.django_db
@pytest.mark.parametrize("logged_in", (False, True))
def test_modify_session(client, settings, logged_in: bool):
    if logged_in:
        user = User.objects.create_superuser("user", "", "secret")
        client.force_login(user)

    read_session_response = client.get("/read-session/", HTTP_USER_AGENT=USER_AGENT)
    read_session_data = json.loads(read_session_response.content.decode("UTF-8"))
    print("read_session_response", read_session_data)

    modify_session_response = client.get("/modify-session/", HTTP_USER_AGENT=USER_AGENT)
    modify_session_data = json.loads(modify_session_response.content.decode("UTF-8"))
    print("modify_session_data", modify_session_data)

    new_read_session_response = client.get("/read-session/", HTTP_USER_AGENT=USER_AGENT)
    new_read_session_data = json.loads(new_read_session_response.content.decode("UTF-8"))

    assert read_session_data != modify_session_data

    if logged_in:
        assert new_read_session_data == modify_session_data
    else:
        assert new_read_session_data != modify_session_data

    assert "ping" not in read_session_data
    assert "ping" in modify_session_data
    assert "ping" in new_read_session_data
    assert SESSION_IP_KEY not in read_session_data

    if logged_in:
        assert SESSION_IP_KEY in modify_session_data
    else:
        assert SESSION_IP_KEY not in modify_session_data
    assert SESSION_IP_KEY in new_read_session_data
    assert SESSION_DEVICE_KEY not in read_session_data
    if logged_in:
        assert SESSION_DEVICE_KEY in modify_session_data
    else:
        assert SESSION_DEVICE_KEY not in modify_session_data
    assert SESSION_DEVICE_KEY in new_read_session_data

    assert modify_session_data["ping"] == "pong"
    assert new_read_session_data["ping"] == "pong"
    assert new_read_session_data[SESSION_IP_KEY] == "127.0.0.1"
    assert new_read_session_data[SESSION_DEVICE_KEY] == DEVICE

    if logged_in:
        assert SESSION_KEY in read_session_data
        assert SESSION_KEY in modify_session_data
        assert SESSION_KEY in new_read_session_data

        assert (
            read_session_data[SESSION_KEY]
            == modify_session_data[SESSION_KEY]
            # == new_read_session_data[SESSION_KEY]
            == str(user.id)
        )

    assert settings.SESSION_COOKIE_NAME in client.cookies
    session = Session.objects.get(pk=client.cookies[settings.SESSION_COOKIE_NAME].value)
    assert session.device == DEVICE
    assert session.ip == "127.0.0.1"
    if logged_in:
        assert session.user == user
    else:
        assert session.user is None



import os
import tarfile
import urllib.request
from pathlib import Path

from django.conf import settings
from django.core.management.base import BaseCommand, CommandParser
from django.utils.http import urlencode

DEFAULT_MAXMIND_GEOIP_DOWNLOAD_BASE_URL = "https://download.maxmind.com/app/geoip_download"


class Command(BaseCommand):
    help = "Download or update existing GeoIP2 database files."

    def add_arguments(self, parser: CommandParser) -> None:
        parser.add_argument("-k", "--maxmind-license-key", required=True)
        parser.add_argument(
            "-u",
            "--maxmind-geoip-download-base-url",
            default=DEFAULT_MAXMIND_GEOIP_DOWNLOAD_BASE_URL,
            help=f"Base URL for downloading GeoIP2 database files. The default is {DEFAULT_MAXMIND_GEOIP_DOWNLOAD_BASE_URL}.",
        )

    def handle(
        self,
        maxmind_license_key: str,
        maxmind_geoip_download_base_url: str,
        *args,
        **options,
    ) -> None:
        if not (db_path := getattr(settings, "GEOIP_PATH", None)):
            return self.stderr.write("GEOIP_PATH is not set in settings.py. Aborting.")

        db_path = Path(db_path)

        if not db_path.exists():
            db_path.mkdir(parents=True)

        for basename, url in (
            (
                "GeoLite2-City.tar.gz",
                self.get_download_url(
                    maxmind_geoip_download_base_url,
                    maxmind_license_key,
                    "GeoLite2-City",
                ),
            ),
            (
                "GeoLite2-Country.tar.gz",
                self.get_download_url(
                    maxmind_geoip_download_base_url,
                    maxmind_license_key,
                    "GeoLite2-Country",
                ),
            ),
        ):
            fp = db_path / basename
            self.stdout.write(f"Downloading {basename} to {fp}...")
            urllib.request.urlretrieve(url, str(fp))  # nosec
            self.stdout.write(f"Downloaded {fp}. Extracting...")
            self.extract_tar(db_path, fp)
            self.stdout.write(f"Extracted {fp}. Clealing...")
            fp.unlink()
            self.stdout.write("Cleaned.")

    @staticmethod
    def get_download_url(maxmind_geoip_download_base_url: str, maxmind_license_key: str, edition: str) -> str:
        return f"{maxmind_geoip_download_base_url}?{urlencode({'edition_id': edition, 'license_key': maxmind_license_key, 'suffix': 'tar.gz'})}"

    def extract_tar(self, db_path: Path, tar_path: Path) -> None:
        with tarfile.open(tar_path) as tarball:
            for tarinfo in tarball:
                if tarinfo.name.endswith(".mmdb"):
                    tarinfo.name = os.path.basename(tarinfo.name)
                    tarball.extract(tarinfo, path=str(db_path))



from django.contrib import admin
from django.http import HttpRequest, JsonResponse
from django.urls import path


def read_session(request: HttpRequest) -> JsonResponse:
    return JsonResponse(dict(request.session))


def modify_session(request: HttpRequest) -> JsonResponse:
    request.session["ping"] = "pong"
    return read_session(request)


urlpatterns = [
    path("read-session/", read_session),
    path("modify-session/", modify_session),
    path("admin/", admin.site.urls),
]



from django.apps import AppConfig
from django.conf import settings

if "admin_auto_filters" not in settings.INSTALLED_APPS:
    settings.INSTALLED_APPS.append("admin_auto_filters")


class DjangoSessionsNgConfig(AppConfig):
    name = "django_user_sessions_ng"
    label = "django_user_sessions_ng"
    verbose_name = "Django User Sessions NG"
    models_module = "django_user_sessions_ng.models"



from pathlib import Path

DEBUG = True
ALLOWED_HOSTS = ("*",)
BASE_DIR = Path(__file__).resolve().parent.parent
SECRET_KEY = "qwerty"
INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.admin",
    "django_user_sessions_ng",
]
MIDDLEWARE = (
    "django_user_sessions_ng.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
)
ROOT_URLCONF = "tests.urls"
CACHES = {"default": {"BACKEND": "django.core.cache.backends.locmem.LocMemCache"}}
DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": "db.sqlite3"}}
SESSION_ENGINE = "django_user_sessions_ng.backends.cached_db"
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]
STATIC_URL = "/static/"

GEOIP_PATH = BASE_DIR / "GeoLite"



from datetime import timedelta

import pytest
from django.apps import apps
from django.conf import settings
from django.contrib import auth
from django.contrib.sessions.backends.base import CreateError
from django.utils.timezone import now

from django_user_sessions_ng import SESSION_DEVICE_KEY, SESSION_IP_KEY
from django_user_sessions_ng.backends import SessionBaseChild

Session = apps.get_model("django_user_sessions_ng", "Session")


@pytest.fixture
def session_store(session_store_class) -> SessionBaseChild:
    return session_store_class(ip="127.0.0.1", device="Test Device")


def test_untouched_session_store(session_store):
    assert not session_store.modified
    assert not session_store.accessed
    assert session_store.is_empty()


def test_session_store(session_store):
    session_store["ping"] = "pong"
    assert session_store.accessed
    assert session_store.modified
    assert not session_store.is_empty()
    assert "ping" in session_store
    assert session_store.pop("ping") == "pong"
    assert "ping" not in session_store
    assert session_store.get("ping") is None


def test_auth_session_key(session_store):
    assert auth.SESSION_KEY not in session_store
    assert not session_store.modified
    assert session_store.accessed

    session_store.get(auth.SESSION_KEY)
    assert not session_store.modified

    session_store[auth.SESSION_KEY] = 1
    assert session_store.modified


@pytest.mark.django_db
def test_save(session_store, django_user_model):
    django_user_model.objects.create_user(username="test_user")

    session_store[auth.SESSION_KEY] = 1
    session_store.save()

    session = Session.objects.get(pk=session_store.session_key)
    assert session.device == "Test Device"
    assert session.ip == "127.0.0.1"
    assert session.user_id == 1
    assert now() - timedelta(seconds=5) <= session.updated_at <= now()


@pytest.mark.django_db
def test_load_unmodified(session_store_class, session_store, django_user_model):
    django_user_model.objects.create_user(username="test_user")

    session_store[auth.SESSION_KEY] = 1
    session_store.save()
    session_store2 = session_store_class(session_key=session_store.session_key, device="Test Device", ip="127.0.0.1")
    session_store2.load()
    assert session_store2.get(SESSION_DEVICE_KEY) == "Test Device"
    assert session_store2.get(SESSION_IP_KEY) == "127.0.0.1"
    assert session_store2.get(auth.SESSION_KEY) == 1
    assert session_store2.modified is False


@pytest.mark.django_db
def test_load_modified(session_store_class, session_store, django_user_model):
    django_user_model.objects.create_user(username="test_user")

    session_store[auth.SESSION_KEY] = 1
    session_store.save()

    session_store2 = session_store_class(
        session_key=session_store.session_key,
        device="Test Device Changed",
        ip="8.8.8.8",
    )
    session_store2.load()

    assert session_store2.get(SESSION_DEVICE_KEY) == "Test Device"
    assert session_store2.get(SESSION_IP_KEY) == "127.0.0.1"
    assert session_store2.get(auth.SESSION_KEY) == 1
    assert session_store2.modified is True

    session_store2.save()

    assert session_store2.get(SESSION_DEVICE_KEY) == "Test Device Changed"
    assert session_store2.get(SESSION_IP_KEY) == "8.8.8.8"


@pytest.mark.django_db
def test_duplicate_create(session_store_class):
    s1 = session_store_class(session_key="DUPLICATE", device="Test Device", ip="127.0.0.1")
    s1.create()
    s2 = session_store_class(session_key="DUPLICATE", device="Test Device", ip="127.0.0.1")
    s2.create()
    assert s1.session_key != s2.session_key

    s3 = session_store_class(session_key=s1.session_key, device="Test Device", ip="127.0.0.1")
    with pytest.raises(CreateError):
        s3.save(must_create=True)


@pytest.mark.django_db
def test_delete(session_store):
    # session_store persisted, should just return
    session_store.delete()

    # create, then delete
    session_store.create()
    session_key = session_store.session_key
    session_store.delete()

    # non-existing sessions, should not raise
    session_store.delete()
    session_store.delete(session_key)


@pytest.mark.django_db
def test_clear(session_store):
    # clearing the session should clear all non-browser information
    session_store[auth.SESSION_KEY] = 1
    session_store.clear()
    session_store.save()

    session = Session.objects.get(pk=session_store.session_key)
    assert session.user_id is None


def test_import(session_store_class):
    if settings.SESSION_ENGINE.endswith(".cached_db"):
        from django_user_sessions_ng.backends.cached_db import (
            SessionStore as CachedDBBackend,
        )

        assert issubclass(session_store_class, CachedDBBackend)
    elif settings.SESSION_ENGINE.endswith(".db"):
        from django_user_sessions_ng.backends.db import SessionStore as DBBackend

        assert issubclass(session_store_class, DBBackend)
    else:
        assert False, "Unrecognised Session Engine"



from pprint import pformat

from admin_auto_filters.filters import AutocompleteFilterFactory
from django.contrib import admin
from django.db.models import QuerySet
from django.http import HttpRequest
from django.utils.html import format_html
from django.utils.timezone import now

from .models import Session


class IsValidFilter(admin.SimpleListFilter):
    title = "validity"
    parameter_name = "active"

    def lookups(
        self, request: HttpRequest, model_admin: admin.ModelAdmin
    ) -> tuple[tuple[bool, str], tuple[bool, str]]:
        return ((True, "active"), (False, "expired"))

    def queryset(self, request: HttpRequest, queryset: QuerySet) -> QuerySet:
        if self.value() is True:
            return queryset.filter(expire_date__gt=now())
        elif self.value() is False:
            return queryset.filter(expire_date__lte=now())
        else:
            return queryset


@admin.register(Session)
class SessionAdmin(admin.ModelAdmin):
    search_fields = ()
    list_display = (
        "session_key",
        "user",
        "ip",
        "device",
        "location",
        "is_valid",
        "created_at",
        "expire_date",
    )
    sortable_by = (
        "user",
        "ip",
        "device",
        "location",
        "is_valid",
        "created_at",
        "expire_date",
    )
    list_filter = (
        IsValidFilter,
        AutocompleteFilterFactory("user", "user"),
        "created_at",
        "expire_date",
    )
    fields = (
        "session_key",
        "user_id",
        "user",
        "ip",
        "device",
        "location",
        "is_valid",
        "session_data_decoded",
        "created_at",
        "expire_date",
    )
    readonly_fields = (
        "session_key",
        "user_id",
        "user",
        "ip",
        "device",
        "location",
        "is_valid",
        "session_data_decoded",
        "created_at",
        "expire_date",
    )

    @admin.display(boolean=True, description="Is valid")
    def is_valid(self, obj: Session) -> bool:
        return obj.expire_date > now()

    @admin.display(description="Session data (decoded)")
    def session_data_decoded(self, obj: Session) -> str:
        return format_html(
            '<pre style="white-space: pre-wrap; max-width: 800px; display: inline-block; direction: ltr;">{}</pre>',
            pformat(obj.get_decoded()),
        )



import user_agents
from django.conf import settings
from django.contrib.sessions.middleware import (
    SessionMiddleware as DjangoSessionMiddleware,
)
from django.http import HttpRequest
from ipware import get_client_ip


class SessionMiddleware(DjangoSessionMiddleware):
    def process_request(self, request: HttpRequest) -> None:
        session_key = request.COOKIES.get(settings.SESSION_COOKIE_NAME, None)
        ip = get_client_ip(request)[0]
        user_agent = request.headers.get("User-Agent", None)
        device = str(user_agents.parse(user_agent)) if user_agent else None
        request.session = self.SessionStore(ip=ip, device=device, session_key=session_key)



from django.apps import apps
from django.contrib import auth
from django.contrib.sessions.backends.db import SessionStore as DjangoSessionStore
from django.utils import timezone

from django_user_sessions_ng import SESSION_DEVICE_KEY, SESSION_IP_KEY

Session = apps.get_model("django_user_sessions_ng", "Session")


class SessionStore(DjangoSessionStore):
    ip: str | None
    device: str | None

    def __init__(
        self,
        session_key: str | None = None,
        ip: str | None = None,
        device: str | None = None,
    ):
        super().__init__(session_key)
        self.ip = ip
        self.device = device[:255] if device else None

    @classmethod
    def get_model_class(cls) -> type[Session]:  # type: ignore
        return Session

    @property
    def key_salt(self) -> str:
        return "django_user_sessions_ng." + self.__class__.__qualname__

    @property
    def user_id(self) -> str | None:
        return self.get(auth.SESSION_KEY)

    def create_model_instance(self, data: dict) -> Session:  # type: ignore
        now = timezone.now()
        return self.model(
            session_key=self._get_or_create_session_key(),
            session_data=self.encode(data),
            expire_date=self.get_expiry_date(),
            user_id=self.user_id,
            ip=self.ip,
            device=self.device,
            created_at=now,
            updated_at=now,
        )

    def load(self) -> dict:
        data: dict = super().load()
        device = data.get(SESSION_DEVICE_KEY)
        ip = data.get(SESSION_IP_KEY)
        if device != self.device or ip != self.ip:
            self.modified = True
        return data

    def save(self, must_create: bool = False) -> Session:  # type: ignore
        if self.get(SESSION_DEVICE_KEY) != self.device:
            self[SESSION_DEVICE_KEY] = self.device
        if self.get(SESSION_IP_KEY) != self.ip:
            self[SESSION_IP_KEY] = self.ip

        return super().save(must_create)



import random
import string

import pytest
from django.contrib import auth

from django_user_sessions_ng import SESSION_DEVICE_KEY, SESSION_IP_KEY
from django_user_sessions_ng.models import Session


def get_random_string(length: int) -> str:
    return "".join(random.choices(string.ascii_letters, k=length))


@pytest.mark.django_db
def test_get_decoded(session_store_class, django_user_model):
    django_user_model.objects.create_user(username="test_user")

    store = session_store_class(device="Test Device", ip="127.0.0.1")
    store[auth.SESSION_KEY] = 1
    store["foo"] = "bar"
    store.save()

    session = Session.objects.get(pk=store.session_key)
    assert session.get_decoded() == {
        "foo": "bar",
        auth.SESSION_KEY: 1,
        SESSION_IP_KEY: "127.0.0.1",
        SESSION_DEVICE_KEY: "Test Device",
    }


@pytest.mark.django_db
def test_very_long_device(session_store_class):
    device = get_random_string(256)
    store = session_store_class(device=device, ip="127.0.0.1")
    store.save()

    session = Session.objects.get(pk=store.session_key)
    assert session.device == device[:255]


@pytest.mark.django_db
def test_delete(session_store_class):
    """
    Session.delete should delete session from both DB and cache
    """
    store = session_store_class(device="Test Device", ip="127.0.0.1")
    store.create()
    session_key = store.session_key

    session = Session.objects.get(pk=session_key)
    session.delete()

    assert not store.exists(session_key)


@pytest.mark.django_db
def test_bulk_delete_from_both_cache_and_db(session_store_class):
    s1 = session_store_class(device="Test Device 1", ip="127.0.0.1")
    s1.create()
    s2 = session_store_class(device="Test Device 1", ip="127.0.0.1")
    s2.create()
    s3 = session_store_class(device="Test Device 2", ip="127.0.0.1")
    s3.create()
    assert s1.exists(s1.session_key)
    assert s2.exists(s2.session_key)
    assert s3.exists(s3.session_key)
    Session.objects.filter(device="Test Device 1").delete()
    assert not s1.exists(s1.session_key)
    assert not s2.exists(s2.session_key)
    assert s3.exists(s3.session_key)



from datetime import timedelta

import pytest
from django.apps import apps
from django.core.management import call_command
from django.utils import timezone

Session = apps.get_model("django_user_sessions_ng", "Session")


@pytest.mark.django_db
def test_can_call():
    Session.objects.create(
        session_key="s1",
        expire_date=timezone.now() + timedelta(hours=1),
        ip="127.0.0.1",
    )
    Session.objects.create(
        session_key="s2",
        expire_date=timezone.now() - timedelta(hours=1),
        ip="127.0.0.1",
    )
    assert Session.objects.count() == 2
    call_command("clearsessions")
    assert Session.objects.count() == 1



import pytest
from django.apps import apps
from django.conf import settings

Session = apps.get_model("django_user_sessions_ng", "Session")
SESSION_ENGINES = (
    "django_user_sessions_ng.backends.db",
    "django_user_sessions_ng.backends.cached_db",
)


@pytest.fixture(autouse=True, params=SESSION_ENGINES)
def session_store_class(request):
    settings.SESSION_ENGINE = request.param
    return Session.get_session_store_class()
